/**
  \file RUFunc.cpp
 */


/*
<func2doc>

</func2doc>
 */


#include <openfluid/base.hpp>
#include <openfluid/core.hpp>
#include <openfluid/core/BooleanValue.hpp>

// =====================================================================
// =====================================================================


DECLARE_PLUGIN_HOOKS;


// =====================================================================
// =====================================================================


BEGIN_SIGNATURE_HOOK
DECLARE_SIGNATURE_ID("training.road.traffic");
DECLARE_SIGNATURE_NAME("training.road.traffic");
DECLARE_SIGNATURE_DESCRIPTION("Road Unit (RU) function transfert and stockage for cars");

DECLARE_SIGNATURE_VERSION("12.03");
DECLARE_SIGNATURE_SDKVERSION;
DECLARE_SIGNATURE_STATUS(openfluid::base::EXPERIMENTAL);

DECLARE_SIGNATURE_DOMAIN("training");
DECLARE_SIGNATURE_PROCESS("");
DECLARE_SIGNATURE_METHOD("");
DECLARE_SIGNATURE_AUTHORNAME("Rabotin M.");
DECLARE_SIGNATURE_AUTHOREMAIL("rabotin@supagro.inra.fr");


DECLARE_USED_VAR("training.TLU.S.state","TLU","traffic light unit state","");
DECLARE_USED_VAR("training.RU.car.source","RU","number of cars in source","-");
DECLARE_PRODUCED_VAR("training.RU.S.stock","RU","number of cars stocked on RU","");
DECLARE_PRODUCED_VAR("training.RU.T.transfert","RU","number of cars transfered on RU","");
DECLARE_USED_INPUTDATA("stockini","RU","","-");
DECLARE_USED_INPUTDATA("transfervalue","RU","","-");








END_SIGNATURE_HOOK


// =====================================================================
// =====================================================================


/**

 */
class RUFunction : public openfluid::base::PluggableFunction
{
  private:
  bool m_UseTLU;
  bool m_UseTLUState;

  public:


  RUFunction(): PluggableFunction()
  {


  }


  // =====================================================================
  // =====================================================================


  ~RUFunction()
  {


  }


  // =====================================================================
  // =====================================================================


  bool initParams(openfluid::core::FuncParamsMap_t /*Params*/)
  {


    return true;
  }

  // =====================================================================
  // =====================================================================


  bool prepareData()
  {


    return true;
  }


  // =====================================================================
  // =====================================================================


  bool checkConsistency()
  {


    return true;
  }


  // =====================================================================
  // =====================================================================


  bool initializeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
  {


    return true;
  }

  // =====================================================================
  // =====================================================================

  bool runStep(const openfluid::base::SimulationStatus* SimStatus)
   {

     openfluid::core::Unit* RU;
     openfluid::core::Unit* UpRU;
     openfluid::core::Unit* UpTLU;
     openfluid::core::DoubleValue StockValue;
     openfluid::core::DoubleValue StockValueSource;
     openfluid::core::DoubleValue Transfert;
     openfluid::core::DoubleValue Capacity;
     openfluid::core::DoubleValue UpTransfert;
     openfluid::core::DoubleValue TmpValue;
     bool TLUState;
     std::list<openfluid::core::Unit*>* UpRUsList;
     std::list<openfluid::core::Unit*>* UpTLUsList;
     std::list<openfluid::core::Unit*>::iterator UpRUiter;


     DECLARE_UNITS_ORDERED_LOOP(1);

     BEGIN_UNITS_ORDERED_LOOP(1,"RU",RU);


     if(SimStatus->isFirstStep())
     {
       OPENFLUID_GetInputData(RU,"stockini",&StockValue);

       Transfert=0;
     }
     else
     {
       UpTLUsList = RU->getFromUnits("TLU");
       TLUState=1;
       if (UpTLUsList != NULL)
       {
         UpTLU = *UpTLUsList->begin();
         OPENFLUID_GetVariable(UpTLU,("training.TLU.S.state"),SimStatus->getCurrentStep(),&TLUState);
       }

       UpTransfert=0;
       UpRUsList = RU->getFromUnits("RU");

       if (UpRUsList != NULL)
       {
         for(UpRUiter=UpRUsList->begin(); UpRUiter != UpRUsList->end(); UpRUiter++)
         {
           UpRU = *UpRUiter;
           OPENFLUID_GetVariable(UpRU,("training.RU.T.transfert"),SimStatus->getCurrentStep(),&TmpValue);
           UpTransfert=UpTransfert+TmpValue;
         }
       }

       OPENFLUID_GetVariable(RU,"training.RU.S.stock",SimStatus->getCurrentStep()-1,&StockValue);
       OPENFLUID_GetInputData(RU,"transfervalue",&Capacity);

       if (TLUState==true)
       {
         Transfert=std::min(StockValue,Capacity);
       }

       else if (TLUState==false )
       {
         Transfert=0;
       }
       StockValue=StockValue+UpTransfert-Transfert;
     }
     OPENFLUID_GetVariable(RU,("training.RU.car.source"), SimStatus->getCurrentStep(), &StockValueSource);

     StockValue=StockValue+StockValueSource;
     OPENFLUID_AppendVariable(RU,"training.RU.S.stock",StockValue);
     OPENFLUID_AppendVariable(RU,"training.RU.T.transfert",Transfert);

     END_LOOP;


     return true;
   }


  // =====================================================================
  // =====================================================================


  bool finalizeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
  {


    return true;
  }

};


// =====================================================================
// =====================================================================


DEFINE_FUNCTION_HOOK(RUFunction);

