/**
  \file TLUFunc.cpp
*/


/*
<func2doc>

</func2doc>
*/


#include <openfluid/base.hpp>
#include <openfluid/core.hpp>
#include <openfluid/core/BooleanValue.hpp>

// =====================================================================
// =====================================================================


DECLARE_PLUGIN_HOOKS;


// =====================================================================
// =====================================================================


BEGIN_SIGNATURE_HOOK
  DECLARE_SIGNATURE_ID("training.trafficlight.state");
  DECLARE_SIGNATURE_NAME("training.trafficlight.state");
  DECLARE_SIGNATURE_DESCRIPTION("Traffic Light Unit (TLU) function state");

  DECLARE_SIGNATURE_VERSION("12.03");
  DECLARE_SIGNATURE_SDKVERSION;
  DECLARE_SIGNATURE_STATUS(openfluid::base::EXPERIMENTAL);

  DECLARE_SIGNATURE_DOMAIN("training");
  DECLARE_SIGNATURE_PROCESS("");
  DECLARE_SIGNATURE_METHOD("");
  DECLARE_SIGNATURE_AUTHORNAME("Rabotin M.");
  DECLARE_SIGNATURE_AUTHOREMAIL("rabotin@supagro.inra.fr");


  DECLARE_PRODUCED_VAR("training.TLU.S.state","TLU","traffic light unit state ","");
  DECLARE_PRODUCED_VAR("training.TLU.T.changeTime","TLU","traffic light unit state changing time","");
  DECLARE_REQUIRED_INPUTDATA("state","TLU","","-");
  DECLARE_REQUIRED_INPUTDATA("duration","TLU","","-");

  
  
  
  
  

END_SIGNATURE_HOOK


// =====================================================================
// =====================================================================


/**

*/
class TLUFunction : public openfluid::base::PluggableFunction
{
  private:

  
  public:

  
    TLUFunction(): PluggableFunction()
    {
  
  
    }
  
  
    // =====================================================================
    // =====================================================================
  
  
    ~TLUFunction()
    {
  
  
    }
  
  
    // =====================================================================
    // =====================================================================
  
  
    bool initParams(openfluid::core::FuncParamsMap_t /*Params*/)
    {
  
  
      return true;
    }
  
    // =====================================================================
    // =====================================================================
  
  
    bool prepareData()
    {
  
  
      return true;
    }
  
  
    // =====================================================================
    // =====================================================================
  
  
    bool checkConsistency()
    {
  
  
      return true;
    }
  
  
    // =====================================================================
    // =====================================================================
  
  
    bool initializeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
    {
  
  
      return true;
    }
  
    // =====================================================================
    // =====================================================================
  
  
    bool runStep(const openfluid::base::SimulationStatus* SimStatus)
     {
       openfluid::core::Unit * pTLU;
       openfluid::core::StringValue PeriodString;
       openfluid::core::IntegerValue PeriodInt;
       openfluid::core::IntegerValue NextPeriod;
       openfluid::core::StringValue State;
       bool StateBool;

       DECLARE_UNITS_ORDERED_LOOP(1);

       BEGIN_UNITS_ORDERED_LOOP(1,"TLU",pTLU);


       OPENFLUID_GetInputData(pTLU,"duration",PeriodString);
       PeriodString.toIntegerValue(PeriodInt);

       OPENFLUID_GetInputData(pTLU,"state",State);
       State.toBoolean(StateBool);

       if(SimStatus->isFirstStep())
       {
         NextPeriod=PeriodInt;
         OPENFLUID_AppendVariable(pTLU,"training.TLU.T.changeTime",NextPeriod);
         OPENFLUID_AppendVariable(pTLU,"training.TLU.S.state",StateBool);
       }
       else
       {
         OPENFLUID_GetVariable(pTLU,"training.TLU.T.changeTime",SimStatus->getCurrentStep()-1,&NextPeriod);
         OPENFLUID_GetVariable(pTLU,"training.TLU.S.state",SimStatus->getCurrentStep()-1,StateBool);

         if(NextPeriod==SimStatus->getCurrentStep())
         {
           StateBool=1-StateBool;
           NextPeriod=NextPeriod+PeriodInt;
         }
         OPENFLUID_AppendVariable(pTLU,"training.TLU.T.changeTime",NextPeriod);
         OPENFLUID_AppendVariable(pTLU,"training.TLU.S.state",StateBool);
       }

       END_LOOP



       return true;
     }
  
    // =====================================================================
    // =====================================================================
  
  
    bool finalizeRun(const openfluid::base::SimulationInfo* /*SimInfo*/)
    {
  
  
      return true;
    }

};


// =====================================================================
// =====================================================================


DEFINE_FUNCTION_HOOK(TLUFunction);

